/*
 * GraphBase.h
 *
 *   Created on: Jun 19, 2014
 *       Author: David Rosca
 *  Modified by: Jan Broz
 */

#ifndef GRAPH_BASE_H_
#define GRAPH_BASE_H_

#include <object/ObjectBase.h>

#include "GraphTypes.h"
#include "common/Macros.h"

#include <core/xmlApi.hpp>

/* zvazit zmeny:
 *  - ohodnoceny/neohodnoceny graf jako trida
 *     aby bylo mozne u grafu vstupujiciho do algoritmu overit existujici vahy behem kompilace a nehazet vyjimky
 *  - uzly a hrany identifikovane unikatnim celociselnym ID misto sami sebou (objektem a hash-codem)
 *     toto by velmi vyrazne zlepsilo efektivitu operaci a algoritmu za cenu drobneho snizeni matematicke presnosti
 */

namespace graph {

/**
 * Represents symbol in an alphabet.
 */
class GraphBase : public object::ObjectBase {

 public:

	virtual ~GraphBase() noexcept {}

	virtual GraphBase * clone ( ) const & = 0;
	virtual GraphBase * clone() && = 0;

	virtual GRAPH_TYPE getType() const = 0;
	virtual REPRESENTATION getRepresentation() const = 0;
	virtual std::string className() const = 0;

	virtual std::string toString() const = 0;

};

} // namespace graph

namespace core {

template < typename Type >
struct xmlApi < Type, typename std::enable_if < std::is_base_of < graph::GraphBase, Type >::value >::type > {
	static Type parse ( ext::deque < sax::Token >::iterator & input ) {
		return Type::parse ( input );
	}

	static bool first ( const ext::deque < sax::Token >::const_iterator & input ) {
		return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName() );
	}

	static std::string xmlTagName ( ) {
		return Type::getXmlTagName();
	}

	static void compose ( ext::deque < sax::Token > & output, const Type & data ) {
		Type::compose ( output, data );
	}

};

} /* namespace core */

#endif // GRAPH_BASE_H_
