/*
 * GraphFromXMLParser.h
 *
 *   Created on: Jun 19, 2014
 *       Author: David Rosca
 */

#ifndef GRAPH_FROM_XML_PARSER_H_
#define GRAPH_FROM_XML_PARSER_H_

#include <sax/FromXMLParserHelper.h>
#include "../GraphClasses.h"
#include <alphabet/SymbolFeatures.h>
#include "Node.h"

namespace graph {

// Parser used to get Graph from XML parsed into list of tokens.
class GraphFromXMLParser {
public:
	static REPRESENTATION parseRepresentation(ext::deque<sax::Token>::iterator& input);
	static Node parseNode(ext::deque<sax::Token>::iterator& input);
	static DirectedEdge parseDirectedEdge(ext::deque<sax::Token>::iterator& input);
	static UndirectedEdge parseUndirectedEdge(ext::deque<sax::Token>::iterator& input);
	static int parseValue(ext::deque<sax::Token>::iterator& input);

	template<typename T>
	static void parseNodes(ext::deque<sax::Token>::iterator& input, T &graph);
	static void parseDirectedEdges(ext::deque<sax::Token>::iterator& input, DirectedGraph &graph);
	static void parseUndirectedEdges(ext::deque<sax::Token>::iterator& input, UndirectedGraph &graph);

	template<typename T>
	static void parseNodeValues(ext::deque<sax::Token>::iterator& input, T &graph);
	static void parseDirectedEdgeValues(ext::deque<sax::Token>::iterator& input, DirectedGraph &graph);
	static void parseUndirectedEdgeValues(ext::deque<sax::Token>::iterator& input, UndirectedGraph &graph);
};

template<typename T>
void GraphFromXMLParser::parseNodes(ext::deque<sax::Token>::iterator& input, T &graph) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "nodes");

	while (sax::FromXMLParserHelper::isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		graph.addNode(parseNode(input));
	}

	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "nodes");
}

template<typename T>
void GraphFromXMLParser::parseNodeValues(ext::deque<sax::Token>::iterator& input, T &graph) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "nodevalues");

	while (sax::FromXMLParserHelper::isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		graph::Node node = parseNode(input);
		int value = parseValue(input);

		graph.setNodeValue(std::move(node), value);
	}

	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "nodevalues");
}

} // namespace graph

#endif // GRAPH_FROM_XML_PARSER_H_
