/*
 * Node.cpp
 *
 *   Created on: Jun 19, 2014
 *       Author: David Rosca
 */

#include "Node.h"

#include <sstream>

namespace graph {

Node::Node(label::Label name)
	: m_name(std::move(name))
{
}

Node::Node(int number)
	: m_name(label::Label(number))
{
}

Node::Node(char character)
	: m_name(label::Label(character))
{
}

Node::Node(const std::string & name)
	: m_name(label::Label(name))
{
}

Node * Node::clone ( ) const &
{
	return new Node(*this);
}

Node * Node::clone() &&
{
	return new Node(std::move(*this));
}

const label::Label & Node::getName() const
{
	return m_name;
}

bool Node::operator < (const Node & other) const {
	return this->compare(other) < 0;
}

bool Node::operator == (const Node & other) const {
	return this->compare(other) == 0;
}

bool Node::operator != (const Node & other) const {
	return !(*this == other);
}

int Node::compare(const Node & other) const
{
	return m_name.getData().compare(other.m_name.getData());
}

void Node::operator<<(std::ostream & out) const
{
	dump(out);
}

std::ostream & operator<<(std::ostream & out, const Node & node)
{
	node.dump(out);
	return out;
}

Node::operator std::string() const
{
	std::stringstream ss;
	ss << *this;
	return ss.str();
}

void Node::dump(std::ostream & os) const
{
	os << "(Node " << m_name << ")";
}

} // namespace graph

