/*
 * DirectedEdge.cpp
 *
 *   Created on: Jun 19, 2014
 *       Author: David Rosca
 */

#include "DirectedEdge.h"
#include <object/Void.h>

#include <sstream>

namespace graph {

DirectedEdge::DirectedEdge(Node from, Node to)
	: DirectedEdge(std::move(from), std::move(to), label::Label('d'))
{
}

DirectedEdge::DirectedEdge(Node from, Node to, label::Label name)
	: m_from(std::move(from))
	, m_to(std::move(to))
	, m_name(std::move(name))
{
}

DirectedEdge::DirectedEdge(Node from, Node to, int number)
	: DirectedEdge(std::move(from), std::move(to), label::Label(number))
{
}

DirectedEdge::DirectedEdge(Node from, Node to, char character)
	: DirectedEdge(std::move(from), std::move(to), label::Label(character))
{
}

DirectedEdge::DirectedEdge(Node from, Node to, std::string name)
	: DirectedEdge(std::move(from), std::move(to), label::Label(name))
{
}

DirectedEdge * DirectedEdge::clone ( ) const &
{
	return new DirectedEdge(*this);
}

DirectedEdge * DirectedEdge::clone() &&
{
	return new DirectedEdge(std::move(*this));
}

const Node & DirectedEdge::getFromNode() const
{
	return m_from;
}

const Node & DirectedEdge::getToNode() const
{
	return m_to;
}

const label::Label & DirectedEdge::getName() const
{
	return m_name;
}

bool DirectedEdge::operator < (const DirectedEdge & other) const {
	return this->compare(other) < 0;
}

bool DirectedEdge::operator > (const DirectedEdge & other) const {
	return this->compare(other) > 0;
}

bool DirectedEdge::operator == (const DirectedEdge & other) const {
	return this->compare(other) == 0;
}

bool DirectedEdge::operator != (const DirectedEdge & other) const {
	return !(*this == other);
}

int DirectedEdge::compare(const DirectedEdge & other) const
{
	int res = m_name.getData().compare(other.m_name.getData());
	if (res != 0) return res;
	res = m_from.compare(other.m_from);
	if (res != 0) return res;
	return m_to.compare(other.m_to);
}

DirectedEdge::operator std::string() const
{
	std::stringstream ss;
	ss << *this;
	return ss.str();
}

std::ostream & operator<<(std::ostream & out, const DirectedEdge & node)
{
	node.dump(out);
	return out;
}

void DirectedEdge::dump(std::ostream & os) const
{
	os << "(DirectedEdge " << m_from << " -> ( " << m_name << " ) " << m_to << ")";
}

} // namespace graph

