/*
 * UndirectedEdge.cpp
 *
 *   Created on: Jun 19, 2014
 *       Author: David Rosca
 */

#include "UndirectedEdge.h"
#include <object/Void.h>

#include <sstream>

namespace graph {

UndirectedEdge::UndirectedEdge(Node first, Node second)
	: UndirectedEdge(std::move(first), std::move(second), label::Label('u'))
{
}

UndirectedEdge::UndirectedEdge(Node first, Node second, label::Label name)
	: m_first(std::move(first))
	, m_second(std::move(second))
	, m_name(std::move(name))
{
	if (this->m_second < this->m_first) {
		Node tmp = std::move(this->m_second);
		this->m_second = std::move(this->m_first);
		this->m_first = std::move(tmp);
	}
}

UndirectedEdge::UndirectedEdge(Node first, Node second, int number)
	: UndirectedEdge(std::move(first), std::move(second), label::Label(number))
{
}

UndirectedEdge::UndirectedEdge(Node first, Node second, char character)
	: UndirectedEdge(std::move(first), std::move(second), label::Label(character))
{
}

UndirectedEdge::UndirectedEdge(Node first, Node second, std::string name)
	: UndirectedEdge(std::move(first), std::move(second), label::Label(name))
{
}

UndirectedEdge * UndirectedEdge::clone ( ) const &
{
	return new UndirectedEdge(*this);
}

UndirectedEdge * UndirectedEdge::clone() &&
{
	return new UndirectedEdge(std::move(*this));
}

const Node & UndirectedEdge::getFirstNode() const
{
	return m_first;
}

const Node & UndirectedEdge::getSecondNode() const
{
	return m_second;
}

const label::Label & UndirectedEdge::getName() const
{
	return m_name;
}

bool UndirectedEdge::operator < (const UndirectedEdge & other) const {
	return this->compare(other) < 0;
}

bool UndirectedEdge::operator > (const UndirectedEdge & other) const {
	return this->compare(other) > 0;
}

bool UndirectedEdge::operator == (const UndirectedEdge & other) const {
	return this->compare(other) == 0;
}

bool UndirectedEdge::operator != (const UndirectedEdge & other) const {
	return !(*this == other);
}

int UndirectedEdge::compare(const UndirectedEdge & other) const
{
	int res = m_name.getData().compare(other.m_name.getData());
	if (res != 0) return res;
	res = m_first.compare(other.m_first);
	if (res != 0) return res;
	return m_second.compare(other.m_second);
}

UndirectedEdge::operator std::string() const
{
	std::stringstream ss;
	ss << *this;
	return ss.str();
}

std::ostream & operator<<(std::ostream & out, const UndirectedEdge & node)
{
	node.dump(out);
	return out;
}

void UndirectedEdge::dump(std::ostream & os) const
{
	os << "(UndirectedEdge " << m_first << " -> ( " << m_name << " ) " << m_second << ")";
}

} // namespace graph

