/*
 * LR0ItemsLabel.cpp
 *
 *  Created on: 29. 4. 2016
 *      Author: Martin Kocicka
 */

#include "LR0ItemsLabel.h"
#include <label/Label.h>
#include <label/UniqueLabel.h>

#include <registration/ValuePrinterRegistration.hpp>

namespace label {

LR0ItemsLabel::LR0ItemsLabel ( grammar::parsing::LR0Items items ) : m_items ( items ) {
}

LR0ItemsLabel::LR0ItemsLabel ( ) {
}

LabelBase* LR0ItemsLabel::clone ( ) const & {
	return new LR0ItemsLabel ( *this );
}

LabelBase* LR0ItemsLabel::clone ( ) && {
	return new LR0ItemsLabel ( std::move ( *this ) );
}

bool LR0ItemsLabel::operator < ( const LR0ItemsLabel & other ) const {
	return this->compare(other) < 0;
}

int LR0ItemsLabel::compare ( const LR0ItemsLabel & other ) const {
	if ( m_items < other.m_items ) {
		return -1;
	} else if ( m_items > other.m_items ) {
		return 1;
	} else {
		return 0;
	}
}

void LR0ItemsLabel::operator>>(std::ostream & out) const {
	out << "(LR0ItemsLabel " << m_items << ")";
}

LR0ItemsLabel::operator std::string ( ) const {
	std::stringstream ss;
	ss << *this;
	return ss.str ( );
}

LabelBase* LR0ItemsLabel::inc ( ) && {
	return new UniqueLabel ( Label ( std::move ( *this ) ), primitive::Integer ( 0 ) );
}

} /* namespace label */

namespace {

static auto valuePrinter = registration::ValuePrinterRegister < label::LR0ItemsLabel > ( );

} /* namespace */
